#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "asm-xml.h"

#include "tools.h"

#define chunkSize 16777216

AXClassContext  classContext;
AXElementClass* docClass;

inline double walker_process_node(AXElement *cur_node, int siblings) {
    AXElement *node;
    int i, size;
    double res = 0;

/*
    printf("Node: %i\nSubnodes: ", cur_node->id);
    for (i=0,node = cur_node->firstChild; node; node = node->nextSibling) {
	printf("%i ", node->id);
    }
    printf("= %i\n", i);
*/
    switch (cur_node->id) {
	case 1:
	    if (siblings<4) return 0;
	    size = 2;
	break;
	case 2:
	case 3:
	    size = 4;
	break;
	default:
	    return 0;
    }

//    printf("Node: %i, size: %i, siblings: %i\n", cur_node->id, size, siblings);

    for (i=0; i<size; i++) {
//	char tmp[256];
//	int len = cur_node->attributes[i].limit - cur_node->attributes[i].begin;
//	printf("%p %p %i\n", cur_node->attributes[i].begin, cur_node->attributes[i].limit, len);
	res += get_value_0(cur_node->attributes[i].begin, cur_node->attributes[i].limit - cur_node->attributes[i].begin);
//	strncpy(tmp, cur_node->attributes[i].begin, len);tmp[len]=0;
//	puts(tmp);
    }
    
    return res;
}

double walker(AXElement *node) {
    int i = 0;
    double res = 0;
    AXElement *cur_node;

    for (cur_node = node; cur_node; cur_node = cur_node->nextSibling) {
	if (++i>3) break;
    }

//    printf("Node %i Subnodes: %i\n", node->id, i);

    for (cur_node = node; cur_node; cur_node = cur_node->nextSibling) {
	if (cur_node->id) {
	    res += walker_process_node(cur_node,i);
	}
	if (cur_node->firstChild) {
	    res += walker(cur_node->firstChild);
	}
    }
    return res;
}


const char *schema_file = NULL;
void initXML(struct TestData *td) {
    struct stat st;
    char *schema = NULL;
    
    if (!strcmp(td->fn, "xmlgen")) {
	schema = ReadFile("../xml.files/generated.schema");
    } else if (schema_file) {
	if (walk_tree) {
	    printf("Asmxml needs custom code for each schema, therefore nothing but xmlgen is supported\n");
	    exit(1);
	}
	schema = ReadFile(schema_file);
    }

    if (!schema) {
        printf("Schema file is not found\n");
        exit(1);
    }
    
    ax_initialize((void*)malloc, (void*)free);

    int res = ax_initializeClassParser(&classContext);
    if (res) {
	printf("Initialization failed: %i\n", res);
	exit(1);
    }
    
    docClass = ax_classFromString(schema+sizeof(unsigned long), &classContext);
    if( docClass == NULL ) {
	printf("Schema parse is failed\n");
	exit(1);
    }

    free(schema);

}



void releaseXML(struct TestData *td) {
    ax_releaseClassParser(&classContext);
}


void parseXML(struct TestData *td, unsigned long iter) {
    AXParseContext parseContext;
    AXElement *root;
    
    int res = ax_initializeParser(&parseContext, chunkSize);
    if( res != 0 ) {
	printf("Parser initialization is failed\n");
	exit(1);
    }

    root = ax_parse(&parseContext, td->xml, docClass, 1);
    if( root == NULL ) {
	printf("Parsing is failed\n");
	exit(1);
    }
    
    if (walk_tree) {
	double res = walker(root);
//	printf("Sum: %lf\n", res);
    }
    
    ax_releaseParser(&parseContext);
}

int main(int argc, char *argv[]) {
    if (argc > 3) schema_file = argv[3];
    
    return Test(argc,argv);
}
