#include <stdio.h>
#include <sys/time.h>
#include "include/util/reader.h"
#include "include/util/filereader.h"
#include "include/util/memoryreader.h"
#include "include/common/defines.h"
#include "include/common/types.h"
#include "include/common/source.h"
#include "include/common/exception.h"
#include "include/xpa/sax/saxparser.h"
#include "include/xpa/sax/saxparserfactory.h"
#include "include/xpa/sax/contenthandler.h"
#include "include/xpa/sax/dtdhandler.h"
#include "include/xpa/sax/entityresolver.h"
#include "include/xpa/sax/lexicalhandler.h"
#include "include/xpa/sax/locator2.h"
#include "include/xpa/sax/saxattribute.h"

#include "tools.h"

using namespace std;
INTEL_XML_NAMESPACE_USE
INTEL_XML_UTIL_NAMESPACE_USE
INTEL_XML_PARSE_SAX_NAMESPACE_USE

class DocHandlers : public ContentHandler/*, public DTDHandler, public EntityResolver, public LexicalHandler*/ {
 public:
    void startElement(unsigned int namespaceTag, unsigned int elemNameCount, const XMLChar* elemName, unsigned int nAttribute, SAXAttribute** attrs);
    void endElement();
    void characters(unsigned int count, const XMLChar* string, bool more);
    void startNamespace(unsigned int newNamespaceTag, unsigned int prefixCount, const XMLChar* prefixString, unsigned int uriCount,const XMLChar* uriString);
    void endNamespace(unsigned int namespaceTag);
    void startDocument();
    void endDocument();
    void pi(unsigned int targetCount, const XMLChar* targetString, unsigned int dataCount, const XMLChar* dataString);
    void setDocumentLocator(Locator* locator);
    void skippedEntity(unsigned int count, const XMLChar* name);
};

void DocHandlers::startElement(unsigned int namespaceTag, unsigned int elemNameCount, const XMLChar* elemName, unsigned int nAttribute, SAXAttribute** attrs) {
    return;
}

void DocHandlers::endElement() {
    return;
}

void DocHandlers::characters(unsigned int count, const XMLChar* string, bool more) {
    return;
}
void DocHandlers::startNamespace(unsigned int newNamespaceTag, unsigned int prefixCount, const XMLChar* prefixString, unsigned int uriCount,const XMLChar* uriString) {
    return;
}
void DocHandlers::endNamespace(unsigned int namespaceTag) {
    return;
}
void DocHandlers::startDocument() {
}
void DocHandlers::endDocument() {
}
void DocHandlers::setDocumentLocator(Locator* locator) {
}
void DocHandlers::skippedEntity(unsigned int count, const XMLChar* name) {
}
void DocHandlers::pi(unsigned int targetCount, const XMLChar* targetString, unsigned int dataCount, const XMLChar* dataString) {
}

class CountHandlers : public DocHandlers {
 private:
    double res_attr[MAX_LEVELS+1];
    double res_sum[MAX_LEVELS+1];
    int res_count[MAX_LEVELS+1];
    int level;

 public:
    double res_total;

    void startDocument();
    void startElement(unsigned int namespaceTag, unsigned int elemNameCount, const XMLChar* elemName, unsigned int nAttribute, SAXAttribute** attrs);
    void endElement();
    void characters(unsigned int count, const XMLChar* string, bool more);
};

void CountHandlers::startDocument() {
    level = 0;
    res_total = 0;
    res_count[0] = 0;
}
void CountHandlers::startElement(unsigned int namespaceTag, unsigned int elemNameCount, const XMLChar* elemName, unsigned int nAttribute, SAXAttribute** attrs) {
    int i;
    double res = 0;

    if (attrs) {
	for (i = 0; i<nAttribute; i++) {
	    if (attrs[i]->valueString) {
	        res = get_value_0(attrs[i]->valueString, attrs[i]->valueCount);
		if (res) break;
	    }
	}
    }

    res_attr[level] = res;
    if (++level == MAX_LEVELS) {
	printf("XML structure is too deep, only %i levels are supported\n", MAX_LEVELS);
	exit(1);
    }
    res_count[level]=0;
}
void CountHandlers::endElement() {
    if (res_count[level] > 3) {
	res_total += res_sum[level];
    }
    
    --level;
    if (res_count[level]) {
	res_sum[level]+=res_attr[level];
    } else {
	res_sum[level]=res_attr[level];
    }
    ++res_count[level];

}
void CountHandlers::characters(unsigned int count, const XMLChar* string, bool more) {
    if ((string)&&(!more)) {
	double res = get_value_0(string,count);
	if (res) res_attr[level-1] = res;
    }
    return;
}


SAXParserFactory* saximpl;
SAXParser *parser;
DocHandlers *handler;

void initXML(struct TestData *td) {
    saximpl = SAXParserFactory::newInstance();
    parser = saximpl->createSAXParser();

    if (walk_tree) {
	handler=new CountHandlers;
    } else {
	handler=new DocHandlers;
    }
    parser->setContentHandler(handler);
    //parser->setDTDHandler(&sax2CoreDumper);
    //parser->setLexicalHandler(&sax2CoreDumper);
    //parser->setEntityResolver(&sax2CoreDumper);
}

void releaseXML(struct TestData *td) {
    delete handler;
    delete parser;
    delete saximpl;
}

void parseXML(struct TestData *td, unsigned long iter) {
    const char id[5]="ID1\0";

    MemoryReader *buffer;
    StreamSource *ss;

    buffer= MemoryReader::createMemoryReader(td->xml,td->xmllen);
    ss = new StreamSource(buffer, id);
//    buffer->setCopyBufToStream(false);
    parser->parse(ss);
    if (walk_tree) {
//	printf("Sum: %lf\n", ((CountHandlers*)handler)->res_total);
    }
    delete ss;
    delete buffer;
}


int main(int argc, char *argv[]) {
    return Test(argc,argv);
}
