#include <stdio.h>
#include <sys/time.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "tools.h"

#if LIBXML_VERSION > 20600
#define _SAX2_BENCHMARK
#endif


void startElement_(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name, const xmlChar **atts) {
}
void endElement_(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name) {
}
void characters_(void *ctx ATTRIBUTE_UNUSED, const xmlChar *ch, int len) {
}

double res_total;
double res_attr[MAX_LEVELS+1];
double res_sum[MAX_LEVELS+1];
int res_count[MAX_LEVELS+1];
int level = 0;

void startDocument_c(void * ctx) {
    level = 0;
    res_total = 0;
    res_count[0] = 0;
}

void startElement_c(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name, const xmlChar **atts) {
    int i;
    double res = 0;

    if (atts) {
	for (i = 0; atts[i]; i++) {
	    res = get_value(atts[i]);
	    if (res) break;
	}
    }

//    printf("%i %lf\n", level, res);
    res_attr[level] = res;
    if (++level == MAX_LEVELS) {
	printf("XML structure is too deep, only %i levels are supported\n", MAX_LEVELS);
	exit(1);
    }
    res_count[level]=0;
    
    return;
}

void endElement_c(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name) {
//    printf("end: %i %s: %i %lf %i %lf\n", level, name, res_count[level-1], res_sum[level-1], res_count[level], res_sum[level]);
    if (res_count[level] > 3) {
	res_total += res_sum[level];
    }
    
    --level;
    if (res_count[level]) {
	res_sum[level]+=res_attr[level];
    } else {
	res_sum[level]=res_attr[level];
    }
    ++res_count[level];

    return;
}

void characters_c(void *ctx ATTRIBUTE_UNUSED, const xmlChar *ch, int len) {
    if (ch) {
	double res = get_value_0(ch,len);
	if (res) res_attr[level-1] = res;
    }
    return;
}


xmlSAXHandler handler = {
    NULL, /* internalSubset */
    NULL, /* isStandalone */
    NULL, /* hasInternalSubset */
    NULL, /* hasExternalSubset */
    NULL, /* resolveEntity */
    NULL, /* getEntity */
    NULL, /* entityDecl */
    NULL, /* notationDecl */
    NULL, /* attributeDecl */
    NULL, /* elementDecl */
    NULL, /* unparsedEntityDecl */
    NULL, /* setDocumentLocator */
    NULL, /* startDocument */
    NULL, /* endDocument */
    startElement_, /* startElement */
    endElement_, /* endElement */
    NULL, /* reference */
    characters_, /* characters */
    NULL, /* ignorableWhitespace */
    NULL, /* processingInstruction */
    NULL, /* comment */
    NULL, /* xmlParserWarning */
    NULL, /* xmlParserError */
    NULL, /* xmlParserError */
    NULL, /* getParameterEntity */
    NULL, /* cdataBlock; */
    NULL, /* externalSubset; */
#ifndef _SAX2_BENCHMARK
    1
#else
    XML_SAX2_MAGIC,    /* XML_SAX1_MAGIC, XML_SAX2_MAGIC */
    NULL,
    NULL, /* startElementNs */
    NULL, /* endElementNs */
    NULL  /* xmlStructuredErrorFunc */
#endif /* _SAX2_BENCHMARK */
};

xmlSAXHandler count_handler = {
    NULL, /* internalSubset */
    NULL, /* isStandalone */
    NULL, /* hasInternalSubset */
    NULL, /* hasExternalSubset */
    NULL, /* resolveEntity */
    NULL, /* getEntity */
    NULL, /* entityDecl */
    NULL, /* notationDecl */
    NULL, /* attributeDecl */
    NULL, /* elementDecl */
    NULL, /* unparsedEntityDecl */
    NULL, /* setDocumentLocator */
    startDocument_c, /* startDocument */
    NULL, /* endDocument */
    startElement_c, /* startElement */
    endElement_c, /* endElement */
    NULL, /* reference */
    characters_c, /* characters */
    NULL, /* ignorableWhitespace */
    NULL, /* processingInstruction */
    NULL, /* comment */
    NULL, /* xmlParserWarning */
    NULL, /* xmlParserError */
    NULL, /* xmlParserError */
    NULL, /* getParameterEntity */
    NULL, /* cdataBlock; */
    NULL, /* externalSubset; */
#ifndef _SAX2_BENCHMARK
    1
#else
    XML_SAX2_MAGIC,    /* XML_SAX1_MAGIC, XML_SAX2_MAGIC */
    NULL,
    NULL, /* startElementNs */
    NULL, /* endElementNs */
    NULL  /* xmlStructuredErrorFunc */
#endif /* _SAX2_BENCHMARK */
};


void initXML(struct TestData *td) {
}

void releaseXML(struct TestData *td) {
    xmlCleanupParser();
}

void parseXML(struct TestData *td, unsigned long iter) {
    if (walk_tree) {
	xmlSAXUserParseMemory(&count_handler,NULL,td->xml,td->xmllen);
//	printf("Sum: %lf\n", res_total);
    } else {
	xmlSAXUserParseMemory(&handler,NULL,td->xml,td->xmllen);
    }
}

int main(int argc, char *argv[]) {
    return Test(argc,argv);
}
