#include <stdio.h>
#include <sys/time.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#ifdef USE_XPATH
# include <libxml/xpath.h>
#endif

#if LIBXML_VERSION > 20600
#define READ_MEMORY
//xmlParserCtxtPtr ctxt = NULL;
#endif

#ifdef READ_MEMORY
#define DTD_VALIDATION_SUPPORTED
#endif

#include "tools.h"


inline double walker_process_node(xmlNode *cur_node) {
    double res;
    xmlChar *value;
    xmlAttr *cur_attr;
    
    if ((cur_node->children)&&(cur_node->children->type == XML_TEXT_NODE)&&(cur_node->children->content)) {
//	puts(cur_node->children->content);

	res = get_value(cur_node->children->content);
	if (res) return res;
    } 

    for (cur_attr = cur_node->properties; cur_attr; cur_attr = cur_attr->next) {
//	value = xmlGetProp(cur_node, cur_attr->name);
	value = (char*)cur_attr->children->content;
	if (value) {
	    res = get_value(value);
//	    xmlFree(value);
	    if (res) return res;
	}
    }
    
    return 0;
}

#ifdef USE_XPATH
inline double xpath_walker(xmlNodeSetPtr nodes) {
    int i;
    double res = 0;
    int size = (nodes) ? nodes->nodeNr : 0;
    
    for(i = 0; i < size; ++i) {
	res += walker_process_node(nodes->nodeTab[i]);
    }
    
    return res;
}
#else
double walker(xmlNode *node) {
    int i = 0;
    double res = 0;
    xmlNode *cur_node;

    for (cur_node = node; cur_node; cur_node = cur_node->next) {
	if((cur_node->type == XML_ELEMENT_NODE)&&(++i>3)) break;
    }
    
    if (i<=3) i = 0;

    for (cur_node = node; cur_node; cur_node = cur_node->next) {
	if (cur_node->type == XML_ELEMENT_NODE) {
	    if (i) {
		res += walker_process_node(cur_node);
	    }
	    if (cur_node->children) {
		res+=walker(cur_node->children);
	    }
	}
    }
    return res;
}
#endif


void initXML(struct TestData *td) {
    xmlInitParser();
#ifdef READ_MEMORY
/*    ctxt = xmlNewParserCtxt();
    if (!ctxt) {
	fprintf(stderr, "Can't create parser context!\n");
	exit(0);
    }*/
#endif
}

void releaseXML(struct TestData *td) {
#ifdef READ_MEMORY
//    if (ctxt) xmlFreeParserCtxt(ctxt);
#endif
    xmlCleanupParser();
}

void parseXML(struct TestData *td, unsigned long iter) {
    int err;
    xmlDocPtr doc;
#ifdef READ_MEMORY
#ifdef DTD_VALIDATION
    doc=xmlReadMemory(td->xml,td->xmllen,"xml",NULL,XML_PARSE_DTDVALID);
#else
    doc=xmlReadMemory(td->xml,td->xmllen,"xml",NULL,0);
#endif
/*    doc=xmlCtxtReadMemory(ctxt,td->xml,td->xmllen,"xml",NULL,XML_PARSE_DTDVALID);
    printf("%u\n",ctxt->valid); */


#else
    doc=xmlParseMemory(td->xml,td->xmllen);
#endif
    if (!doc) {
        printf("Error parsing document!\n");
        exit(0);
    }


    if (walk_tree) {
	double res;
#ifdef USE_XPATH
	xmlXPathContextPtr xpathCtx; 
	xmlXPathObjectPtr xpathObj; 

	xpathCtx = xmlXPathNewContext(doc);
	xpathObj = xmlXPathEvalExpression(walk_xpath, xpathCtx);
	if (xpathObj) {
	    res = xpath_walker(xpathObj->nodesetval);
	    xmlXPathFreeObject(xpathObj);
	    xmlXPathFreeContext(xpathCtx); 
	}
#else
	res = walker(xmlDocGetRootElement(doc));
#endif
//	printf("Sum: %lf\n", res);
    }

    xmlFreeDoc(doc);    

}

int main(int argc, char *argv[]) {
    return Test(argc,argv);
}
