#    Copyright (c) 2008, Robert D. Cameron.
#    Licensed to the public under the Open Software License 3.0.
#    Licensed to International Characters, Inc., under the Academic
#    Free License 3.0.

import codecs;

ord_template = "template<> struct Ord<%s,'%s'> {static uint8_t const value = 0x%x;};\n"

def gen_ASCII(file):
	file.write(ord_template % ('ASCII', "\\0", 0))
	for chr_val in range(0x20,0x7E):
		ch = chr(chr_val)
		if ch == "'" or ch == "\\": ch = "\\" + ch
		file.write(ord_template % ('ASCII', ch, chr_val))

class EncodingFailure(Exception):
	def __init__(self, value):
		self.value = value

def gen_EBCDIC(file):
	file.write(ord_template % ('EBCDIC', "\\0", 0))
	encoder = codecs.getencoder('cp037')
	for chr_val in range(0x20,0x7E):
		ch = chr(chr_val)
		(encoded, src_lgth) = encoder(ch)
		if src_lgth != 1: raise EncodingFailure('input lgth')
		if len(encoded) != 1: raise EncodingFailure('output lgth')
		if ch == "'" or ch == "\\": ch = "\\" + ch
		file.write(ord_template % ('EBCDIC', ch, ord(encoded[0])))

ASCII_EBCDIC_header = r"""/* ASCII_EBCDIC.h
    Copyright (c) 2008, Robert D. Cameron.
    Licensed to the public under the Open Software License 3.0.
    Licensed to International Characters, Inc., under the Academic
    Free License 3.0.
    
    This is a generated file using ASCII_EBCDIC.py.  Do not edit.

*/

#ifndef _MSC_VER
#include <stdint.h>
#endif
#ifdef _MSC_VER
#include "../../lib/stdint.h"
#endif
#include "../xmlmodel.h"

template<CodeUnit_Base C> struct CR;
template<> struct CR<ASCII> {static unsigned char const value = 0x0D;};
template<> struct CR<EBCDIC> {static unsigned char const value = 0x0D;};

template<CodeUnit_Base C> struct LF;
template<> struct LF<ASCII> {static unsigned char const value = 0x0A;};
template<> struct LF<EBCDIC> {static unsigned char const value = 0x25;};

template<CodeUnit_Base C> struct HT;
template<> struct HT<ASCII> {static unsigned char const value = 0x09;};
template<> struct HT<EBCDIC> {static unsigned char const value = 0x05;};

template<CodeUnit_Base C, unsigned char c> struct Ord;

"""
UC2lc_base = "template <unsigned char _> struct UC2lc {static unsigned char const value = _;};\n"
UC2lc_template = "template <> struct UC2lc<'%s'> {static unsigned char const value = '%s';};\n"
lc2UC_base = "template <unsigned char _> struct lc2UC {static unsigned char const value = _;};\n"
lc2UC_template = "template <> struct lc2UC<'%s'> {static unsigned char const value = '%s';};\n"

def gen_UC2lc(f):
	f.write(UC2lc_base)
	for v in range(26):
		f.write(UC2lc_template % (chr(ord('A')+v), chr(ord('a')+v)))

def gen_lc2UC(f):
	f.write(lc2UC_base)
	for v in range(26):
		f.write(lc2UC_template % (chr(ord('a')+v), chr(ord('A')+v)))
	
def gen_ASCII_EBCDIC_h(filename):
	define_sym = filename[:-2] + "_H"
	f = open(filename, 'w')
	f.write("#ifndef %s\n#define %s\n" % (define_sym, define_sym));
	f.write(ASCII_EBCDIC_header + "\n")
	gen_ASCII(f)
	f.write("\n")
	gen_EBCDIC(f)
	f.write("\n")
	gen_UC2lc(f)
	f.write("\n")
	gen_lc2UC(f)
	f.write("\n#endif\n")
	f.close()


if __name__ == "__main__": gen_ASCII_EBCDIC_h('ASCII_EBCDIC.h')
