/*  xml_error.c - Error reporting for XML parsing/validation.
    Copyright (c) 2008, Robert D. Cameron and Dan Lin.
    Licensed to the public under the Open Software License 3.0.
    Licensed to International Characters, Inc., under the Academic
    Free License 3.0.
*/

#include "xml_error.h"

void ShowConstraintError(XML_Constraint errCode) {
	if (errCode < vErr_vc_roottype) {
		fprintf(stderr, "Violation of well-formedness constraint: %s\n", XML_Constraint_Strings[errCode]);
		exit(-1);
	}
	else {
		fprintf(stderr, "Violation of validity constraint: %s\n", XML_Constraint_Strings[errCode]);
		exit(-1);
	}
}

void ShowSyntaxError(XML_NonTerminal errCode) {
	fprintf(stderr, "Syntax error in production: %s\n", XML_NonTerminal_Names[errCode]);
}


void NoEncodingError(const char * msg) {
	fprintf(stderr, "Error : %s\n", msg);
	exit(-1);
}

void EncodingError(const char * msg, unsigned char * encoding, int lgth) {
	fprintf(stderr, "Error : Illegal/unsupported %s encoding of length %i: \"", msg, lgth);
	for (int i = 0; i < lgth; i++) fprintf(stderr, "%c", encoding[i]);
	fprintf(stderr, "\"\n");
	exit(-1);
}

void CharSetValidationError(const char * encoding, int err_pos) {
	fprintf(stderr, "Error: Invalid %s character in input stream at position %i\n", encoding, err_pos);
	exit(-1);
}

void XMLCharacterError(int err_pos) {
	fprintf(stderr, "Illegal control character in XML input stream at position %i\n", err_pos);
	exit(-1);
}

void IncompleteCodeUnitError() {
	fprintf(stderr, "Error: Incomplete code unit at end of file.\n");
	exit(-1);
}

void DeclarationError(int pos) {
	fprintf(stderr, "Parsing error at position %i in XML or Text declaration.\n", pos);
	exit(-1);
}

void ImplementationLimitError(const char * msg) {
	fprintf(stderr, "Fatal implementation limit - %s\n", msg);
	exit(-1);
}

void ContentModelError() {
	fprintf(stderr, "Error: nondeterminism in content model.\n");
	exit(-1);
}

