#include <stdio.h>
#include <sys/time.h>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>

#include "tools.h"

#if defined(XERCES_HAS_CPP_NAMESPACE)
    XERCES_CPP_NAMESPACE_USE
#endif


#ifdef FAST_MEMORY
#define FASTMM_BUF_SIZE 134217728
class FASTMemoryManager : public MemoryManager {
 protected:
    void *buf;
    int cursize;
    int pos;
 public:
    FASTMemoryManager() {
	cursize = FASTMM_BUF_SIZE;
	pos = 0;
	buf = malloc(FASTMM_BUF_SIZE);
	if (!buf) exit(-1);
    }
    
    void * allocate (XMLSize_t size) {
	void *res;

	int mod = size%16;
	if (mod) bsize += 16-mod;

	if (pos + size > cursize) {
	    while (pos + size > cursize) cursize *= 2;
	    buf = realloc(buf, cursize);
	    if (!buf) exit(-1);
	}
	
	res = (char*)buf+pos;
	pos += size;
	return res;
    }
    
    void deallocate (void *p) {
    }
    
    void clean() {
	pos = 0;
    }
    
    ~FASTMemoryManager() {
	free(buf);
    }
} fastmm;
#endif /* FAST_MEMORY */



void initXML(struct TestData *td) {
    XMLPlatformUtils::Initialize();
#ifdef FAST_MEMORY
#endif /* FAST_MEMORY */
}

void releaseXML(struct TestData *td) {
#ifdef FAST_MEMORY
#endif /* FAST_MEMORY */
    XMLPlatformUtils::Terminate();
}

void parseXML(struct TestData *td, unsigned long iter) {
    const char id[5]="ID1\0";
    MemBufInputSource *buffer;
    XercesDOMParser *parser;

#ifdef FAST_MEMORY
    fastmm.clean();
    parser = new XercesDOMParser(NULL,&fastmm);
#else
    parser = new XercesDOMParser;
#endif /* FAST_MEMORY */

    buffer=new MemBufInputSource((XMLByte*)td->xml,td->xmllen,(const char*)&id);
    buffer->setCopyBufToStream(false);
    parser->parse(*buffer);
    delete buffer;
    delete parser;
}


int main(int argc, char *argv[]) {
    return Test(argc,argv);
}
